#ifndef _LABVIEWINTERFACE_h
#define _LABVIEWINTERFACE_h
/***************************************************************
**  Define Constants
**
**  Define directives providing meaningful names for constant values.
***************************************************************/
#include <Wire.h>

#define FIRMWARE_MAJOR 02        
#define FIRMWARE_MINOR 00  

#define __AVR_ATmega328_Nano__

#if defined(__AVR_ATmega328_Nano__)
#define __SERIAL Serial 
#define __WIRE Wire
#else
#define __SERIAL SerialUSB 
#endif

#include <Servo.h>


#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
  #define DEFAULTBAUDRATE 9600    // Defines The Default Serial Baud Rate (This must match the baud rate specifid in LabVIEW)
#else
  #define DEFAULTBAUDRATE 115200 //115200
#endif

#define MODE_DEFAULT 0            // Defines Arduino Modes (Currently Not Used)
#define COMMANDLENGTH 15          // Defines The Number Of Bytes In A Single LabVIEW Command (This must match the packet size specifid in LabVIEW)


// Declare Variables
unsigned char currentCommand[COMMANDLENGTH];    // The Current Command For The Arduino To Process

/***************************************************************
**  syncLV
**
**  Synchronizes with LabVIEW and sends info about the board and firmware (Unimplemented)
**
**  Input:  None
**  Output: None
***************************************************************/
void syncLV();

/***************************************************************
**  setMode
**
**  Sets the mode of the Arduino (Reserved For Future Use)
**
**  Input:  Int - Mode
**  Output: None
***************************************************************/
void setMode(int mode);

/***************************************************************
**  checkForCommand
**
**  Checks for new commands from LabVIEW and processes them if any exists.
**
**  Input:  None
**  Output: 1 - Command received and processed
**          0 - No new command
***************************************************************/
int checkForCommand(void);

/***************************************************************
**  processCommand
**
**  Processes a given command
**
**  Input:  command of COMMANDLENGTH bytes
**  Output: 1 - Command received and processed
**          0 - No new command
***************************************************************/
void processCommand(unsigned char command[]);

/*********************************************************************************
**  analogReadPort
**
**  Reads all 6 analog input ports, builds 8 byte packet, send via RS232.
**
**  Input:  None
**  Output: None
*********************************************************************************/
void analogReadPort();

/***************************************************************
**  checksum_Compute
**
**  Compute Packet Checksum
**
**  Input:  Command Packet
**  Output: Char Checksum Value
***************************************************************/
unsigned char checksum_Compute(unsigned char command[]);

/***************************************************************
**  checksum_Test
**
**  Compute Packet Checksum And Test Against Included Checksum
**
**  Input:  Command Packet
**  Output: 0 If Checksums Are Equal, Else 1
***************************************************************/
int checksum_Test(unsigned char command[]);

#endif
