#include "LabVIEWInterface.h"

// Variables
unsigned int retVal;
int currentMode;
unsigned int freq;
unsigned long duration;
int i2cReadTimeouts = 0;

//Servo
Servo *servos;

volatile uint8_t off = 90;
volatile uint8_t on = 1;
volatile uint8_t offsetangel = 0;
volatile uint8_t servono = 0;

static void test(void);


// Sets the mode of the Arduino (Reserved For Future Use)
void setMode(int mode) {
  currentMode = mode;
}

// Checks for new commands from LabVIEW and processes them if any exists.
int checkForCommand(void) {
  if (__SERIAL.available() >= (int)COMMANDLENGTH) {

    // New Command Ready, Process It
    // Build Command From __SERIALUSB Buffer

    for (int i = 0; i < COMMANDLENGTH; i++) {
      currentCommand[i] = __SERIAL.read();
    }
    processCommand(currentCommand);
    return 1;
  }
  else {
    return 0;
  }
}

// Processes a given command
void processCommand(unsigned char command[]) {
  // Determine Command
  if (command[0] == 0xFF && checksum_Test(command) == 0) {
    switch (command[1]) {

      /***************************************************************
      ** LIFA Maintenance Commands
      ***************************************************************/
      case 0x00:     // Sync Packet
        __SERIAL.print("sync");
        __SERIAL.flush();
        break;
      case 0x01:    // Flush __SERIALUSB Buffer
        __SERIAL.flush();
        break;

      /***************************************************************
      ** Low Level - Digital I/O Commands
      ***************************************************************/
      case 0x02:    // Set Pin As Input Or Output
        pinMode(command[2], command[3]);
        __SERIAL.write('0');
        break;
      case 0x03:    // Write Digital Pin
        digitalWrite(command[2], command[3]);
        __SERIAL.write('0');
        break;
      case 0x06:    // Read Digital Pin
        retVal = digitalRead(command[2]);

        __SERIAL.write(retVal);
        break;
      case 0x07:    // Digital Read Port
        retVal = 0x0000;
        for (int i = 0; i <= 13; i++)
        {
          if (digitalRead(i))
          {
            retVal += (1 << i);
          }
        }
        __SERIAL.write( (retVal & 0xFF));
        __SERIAL.write( (retVal >> 8));
        break;
      /*********************************************************************************
      ** Servos
      *********************************************************************************/
      case 0x17:  // Set Num Servos
        free(servos);
        servos = (Servo*) malloc(command[2] * sizeof(Servo));
        for (int i = 0; i < command[2]; i++)
        {
          servos[i] = Servo();
        }
        if (servos == 0)
        {
          __SERIAL.write('1');
        }
        else
        {
          __SERIAL.write('0');
        }
        break;
      case 0x18:  // Configure Servo
        servos[command[2]].attach(command[3]);
        __SERIAL.write('0');
        break;
      case 0x19:  // Servo Write Angel
        servono = (uint8_t)command[2];
        offsetangel = (uint8_t)command[3];


        test();


        //servos[command[2]].write(command[3]);

        __SERIAL.write('0');
        break;
      case 0x1A:  // Servo Read Angle
        __SERIAL.write(servos[command[2]].read());
        break;
      case 0x1B:  // Servo Write uS Pulse
        servos[command[2]].writeMicroseconds( (command[3] + (command[4] << 8)) );
        __SERIAL.write('0');
        break;
      case 0x1C:  // Servo Read uS Pulse
        retVal = servos[command[2]].readMicroseconds();
        __SERIAL.write ((retVal & 0xFF));
        __SERIAL.write( (retVal >> 8));
        break;
      case 0x1D:  // Servo Detach
        servos[command[2]].detach();
        __SERIAL.write('0');
        break;
      /*********************************************************************************
      ** Low Level - Analog Commands
      *********************************************************************************/
      case 0x08:    // Read Analog Pin
        retVal = analogRead(command[2]);
        Serial.write( (retVal >> 8));
        Serial.write( (retVal & 0xFF));
        break;
      case 0x09:    // Analog Read Port
        analogReadPort();
        break;

      /***************************************************************
      **  I2C
      ***************************************************************/
      case 0x0E:    // Initialize I2C
        //__WIRE.begin();
        __SERIAL.write('0');
        break;
      case 0x0F:    // Send I2C Data
        __WIRE.beginTransmission(command[3]);
        for (int i = 0; i < command[2]; i++) {
#if defined(ARDUINO) && ARDUINO >= 100
          __WIRE.write(command[i + 4]);
#else
          __WIRE.send(command[i + 4]);
#endif

        }
        __WIRE.endTransmission();
        __SERIAL.write('0');
        break;
      case 0x10:    // I2C Read
        i2cReadTimeouts = 0;
        __WIRE.requestFrom(command[3], command[2]);
        while (__WIRE.available() < command[2]) {
          i2cReadTimeouts++;
          if (i2cReadTimeouts > 100) {
            return;
          }
          else {
            delay(1);
          }
        }

        for (int i = 0; i < command[2]; i++) {
#if defined(ARDUINO) && ARDUINO >= 100
          __SERIAL.write(__WIRE.read());
         // __WIRE.read();
         // __SERIAL.write(255);
#else
          __SERIAL.write(__WIRE.receive());
#endif

        }
        break;
      /***************************************************************
      ** Unknown Packet
      ***************************************************************/
      default:      // Default Case
        __SERIAL.flush();
        break;
    }
  }
  else {
    // Checksum Failed, Flush __SERIALUSB Buffer
    __SERIAL.flush();
  }
}

/***************************************************************
**  Functions
***************************************************************/
// Reads all 6 analog input ports, builds 8 byte packet, send via RS232.
void analogReadPort()
{
  // Read Each Analog Pin
  int pin0 = analogRead(A0);
  int pin1 = analogRead(A1);
  int pin2 = analogRead(A2);
  int pin3 = analogRead(A3);
  int pin4 = analogRead(4);
  int pin5 = analogRead(5);

  //Build 8-Byte Packet From 60 Bits of Data Read
  char output0 = (pin0 & 0xFF);
  char output1 = ( ((pin1 << 2) & 0xFC) | ( (pin0 >> 8) & 0x03) );
  char output2 = ( ((pin2 << 4) & 0xF0) | ( (pin1 >> 6) & 0x0F) );
  char output3 = ( ((pin3 << 6) & 0xC0) | ( (pin2 >> 4) & 0x3F) );
  char output4 = ( (pin3 >> 2) & 0xFF);
  char output5 = (pin4 & 0xFF);
  char output6 = ( ((pin5 << 2) & 0xFC) | ( (pin4 >> 8) & 0x03) );
  char output7 = ( (pin5 >> 6) & 0x0F );

  // Write Bytes To Serial Port
  Serial.print(output0);
  Serial.print(output1);
  Serial.print(output2);
  Serial.print(output3);
  Serial.print(output4);
  Serial.print(output5);
  Serial.print(output6);
  Serial.print(output7);
}
// Synchronizes with LabVIEW and sends info about the board and firmware (Unimplemented)
void syncLV() {
  i2cReadTimeouts = 0;

  __SERIAL.begin(DEFAULTBAUDRATE);
  __SERIAL.flush();
}

// Compute Packet Checksum
unsigned char checksum_Compute(unsigned char command[]) {
  unsigned char checksum = 0;
  for (int i = 0; i < (COMMANDLENGTH - 1); i++) {
    checksum += command[i];
  }
  return checksum;
}

// Compute Packet Checksum And Test Against Included Checksum
int checksum_Test(unsigned char command[]) {
  unsigned char checksum = checksum_Compute(command);
  if (checksum == command[COMMANDLENGTH - 1]) {
    return 0;
  }
  else {
    return 1;
  }
}

void changeStateSh1Pin(void) {
  //test();
}

static void test(void) {

  uint8_t angel = 0;

  if ((uint8_t)servos[servono].read() < 90)
    angel = offsetangel;
  else
    angel = offsetangel;

  servos[servono].write(angel);
}

