/***************************************************************
 **
 ** Includes.
 **
***************************************************************/
// Standard includes.  These should always be included.
#include <Wire.h>
#include "LabVIEWInterface.h"

#define ADD_IO_EXPANDER  0x20

#define PWM_SHUTTER_1   5
#define PWM_SHUTTER_2   6
#define PWM_SHUTTER_3   10
#define PWM_SHUTTER_4   11

#define SH1_INT         A0
#define SH2_INT         A1
#define SH3_INT         A2
#define SH4_INT         A3


#define LED1_PWR        12
#define LED2_PWR        9
#define LED3_PWR        8
#define LED4_PWR        7

#define CH1             4
#define CH2             13
#define CH3             3
#define CH4             2

volatile uint8_t count_for_ready_pin = 1;

void pciSetup(byte pin)
{
  *digitalPinToPCMSK(pin) |= bit (digitalPinToPCMSKbit(pin));  // enable pin
  PCIFR  |= bit (digitalPinToPCICRbit(pin)); // clear any outstanding interrupt
  PCICR  |= bit (digitalPinToPCICRbit(pin)); // enable interrupt for the group
}

/***************************************************************
 **  setup()
 **
 **  Initialize the Arduino and setup serial communication.
 **
 **  Input:  None
 **  Output: None
***************************************************************/
void setup() {

  pinMode(CH1, INPUT);
  pinMode(CH2, INPUT);
  pinMode(CH3, INPUT);
  pinMode(CH4, INPUT);

  //pinMode(PWM_SHUTTER_3, OUTPUT);
  //digitalWrite(PWM_SHUTTER_3, LOW);

  /*pinMode(SH1_INT, INPUT);
  pinMode(SH2_INT, INPUT);
  pinMode(SH3_INT, INPUT);
  pinMode(SH4_INT, INPUT);*/

  pinMode(LED1_PWR, OUTPUT);
  digitalWrite(LED1_PWR, LOW);
  pinMode(LED2_PWR, OUTPUT);
  digitalWrite(LED2_PWR, LOW);
  pinMode(LED3_PWR, OUTPUT);
  digitalWrite(LED3_PWR, LOW);
  pinMode(LED4_PWR, OUTPUT);
  digitalWrite(LED4_PWR, LOW);

  //pciSetup(SH1_INT);

  syncLV();

  __WIRE.begin();

  __WIRE.beginTransmission(ADD_IO_EXPANDER);
  // ConfigRegister
  __WIRE.write(0x03);
  // set to output
  __WIRE.write(0x00);
  __WIRE.endTransmission(true);

  __WIRE.beginTransmission(ADD_IO_EXPANDER);
  // OutputRegister
  __WIRE.write(0x01);
  // value
  __WIRE.write(0x8E);
  __WIRE.endTransmission(true);
}


ISR (PCINT1_vect) // handle pin change interrupt for A0 to A5 here
{
  if (count_for_ready_pin == 0) {
    changeStateSh1Pin();
    count_for_ready_pin++;
  }
  else {
    count_for_ready_pin--;
  }
}


// Called when data is available
void serialEvent(void) {
  checkForCommand();
}

/***************************************************************
 **  loop()
 **
 **  The main loop.  This loop runs continuously on the Arduino.  It
 **  receives and processes serial commands from LabVIEW.
 **
 **  Input:  None
 **  Output: None
***************************************************************/
void loop() {
}
